@@ -19,11 +19,11 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemGMPPlugin.h 4052 2007-02-19 16:01:17Z chpe $
+ * $Id: totemGMPPlugin.h 4625 2007-08-29 15:38:18Z hadess $
  */
 
-#ifndef __GMP_PLAYER_H__
-#define __GMP_PLAYER_H__
+#ifndef __GMP_PLUGIN_H__
+#define __GMP_PLUGIN_H__
 
 #include <nsIClassInfo.h>
 
@@ -65,4 +65,4 @@ class totemScriptablePlugin : public totemIGMPPlayer,
     PRUint32 mWindowlessVideo;
 };
 
-#endif /* __GMP_PLAYER_H__ */
+#endif /* __GMP_PLUGIN_H__ */
