@@ -37,7 +37,7 @@
 #include <nsTArray.h>
 #endif /* TOTEM_COMPLEX_PLUGIN */
 
-#include "totem-plugin-viewer-commands.h"
+#include "totem-plugin-viewer-constants.h"
 
 class nsIDOMDocument;
 class nsIDOMElement;
@@ -59,6 +59,7 @@ class totemPlugin {
     /* Interface to scriptable */
 
     nsresult DoCommand (const char *aCommand);
+    nsresult SetVolume (gdouble aVolume);
 
     nsresult SetSrc (const nsACString &aURL);
 
@@ -117,6 +118,12 @@ class totemPlugin {
     static void PR_CALLBACK StopStreamCallback (DBusGProxy  *proxy,
 						void *aData);
 
+    static void PR_CALLBACK TickCallback (DBusGProxy  *proxy,
+    					  guint aTime,
+    					  guint aDuration,
+    					  char *aState,
+					  void *aData);
+
     static void PR_CALLBACK ViewerSetWindowCallback (DBusGProxy *aProxy,
 						     DBusGProxyCall *aCall,
 						     void *aData);
@@ -174,8 +181,12 @@ class totemPlugin {
     nsIURI *mRequestBaseURI;
     nsIURI *mRequestURI;
 
+    /* Stream data */
     NPStream *mStream;
+  public:
     PRUint32 mBytesStreamed;
+    PRUint32 mBytesLength;
+  private:
     PRUint8 mStreamType;
 
     nsCString mMimeType;
@@ -196,6 +207,11 @@ class totemPlugin {
     int mViewerPID;
     int mViewerFD;
 
+  public:
+    PRUint32 mTime;
+    PRUint32 mDuration;
+    TotemStates mState;
+
 #ifdef TOTEM_GMP_PLUGIN
   public:
     nsresult SetURL (const nsACString &aURL);
@@ -253,6 +269,7 @@ class totemPlugin {
   private:
 
     PRUint32 mAutostart : 1;
+    PRUint32 mAutoPlay : 1;
     PRUint32 mCache : 1;
     PRUint32 mCheckedForPlaylist : 1;
     PRUint32 mControllerHidden : 1;
@@ -278,7 +295,6 @@ typedef struct {
   const char *mimetype;
   const char *extensions;
   const char *mime_alias;
-  gboolean    ignore;
 } totemPluginMimeEntry;
 
 #endif /* __TOTEM_PLUGIN_H__ */
