@@ -2,6 +2,9 @@
    Copyright (C) 2002, 2003, 2004, 2005, 2006 Bastien Nocera
    Copyright (C) 2003, 2004 Colin Walters <walters@rhythmbox.org>
 
+   Copyright (C) 2005 Renato Araujo Oliveira Filho - INdT <renato.filho@indt.org.br>
+   for totem_pl_parser_parse_date ()
+
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
    published by the Free Software Foundation; either version 2 of the
@@ -17,13 +20,21 @@
    write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 
-   Author: Bastien Nocera <hadess@hadess.net>
+   Authors: Bastien Nocera <hadess@hadess.net>
+
+            for totem_pl_parser_parse_date ():
+            James Livingston  <doclivingston@gmail.com>
+            William Jon McCann  <mccann@jhu.edu>
+            Renato Araujo Oliveira Filho - INdT <renato.filho@indt.org.br>
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#define _XOPEN_SOURCE
+#include <time.h>
+
 #include <string.h>
 #include <glib.h>
 #include <glib/gi18n-lib.h>
@@ -35,6 +46,7 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+
 #include "totem-pl-parser.h"
 #include "totemplparser-marshal.h"
 #include "totem-disc.h"
@@ -48,6 +60,7 @@
 #include "totem-pl-parser-media.h"
 #include "totem-pl-parser-smil.h"
 #include "totem-pl-parser-pla.h"
+#include "totem-pl-parser-podcast.h"
 #include "totem-pl-parser-lines.h"
 #include "totem-pl-parser-misc.h"
 #include "totem-pl-parser-private.h"
@@ -60,6 +73,8 @@
 #define TEXT_URI_TYPE "text/uri-list"
 #define AUDIO_MPEG_TYPE "audio/mpeg"
 
+#define D(x) if (debug) x
+
 typedef gboolean (*PlaylistIdenCallback) (const char *data, gsize len);
 
 #ifndef TOTEM_PL_PARSER_MINI
@@ -99,8 +114,8 @@ enum {
 /* Signals */
 enum {
 	ENTRY_PARSED,
-	PLAYLIST_START,
-	PLAYLIST_END,
+	PLAYLIST_STARTED,
+	PLAYLIST_ENDED,
 	LAST_SIGNAL
 };
 
@@ -164,19 +179,19 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 			      NULL, NULL,
 			      totemplparser_marshal_VOID__STRING_POINTER,
 			      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_POINTER);
-	totem_pl_parser_table_signals[PLAYLIST_START] =
-		g_signal_new ("playlist-start",
+	totem_pl_parser_table_signals[PLAYLIST_STARTED] =
+		g_signal_new ("playlist-started",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (TotemPlParserClass, playlist_start),
+			      G_STRUCT_OFFSET (TotemPlParserClass, playlist_started),
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE, 1, G_TYPE_STRING);
-	totem_pl_parser_table_signals[PLAYLIST_END] =
-		g_signal_new ("playlist-end",
+			      totemplparser_marshal_VOID__STRING_POINTER,
+			      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_POINTER);
+	totem_pl_parser_table_signals[PLAYLIST_ENDED] =
+		g_signal_new ("playlist-ended",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (TotemPlParserClass, playlist_end),
+			      G_STRUCT_OFFSET (TotemPlParserClass, playlist_ended),
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1, G_TYPE_STRING);
@@ -267,19 +282,11 @@ totem_pl_parser_new (void)
 }
 
 void
-totem_pl_parser_playlist_start (TotemPlParser *parser, const char *playlist_title)
+totem_pl_parser_playlist_end (TotemPlParser *parser, const char *playlist_uri)
 {
 	g_signal_emit (G_OBJECT (parser),
-		       totem_pl_parser_table_signals[PLAYLIST_START],
-		       0, playlist_title);
-}
-
-void
-totem_pl_parser_playlist_end (TotemPlParser *parser, const char *playlist_title)
-{
-	g_signal_emit (G_OBJECT (parser),
-		       totem_pl_parser_table_signals[PLAYLIST_END],
-		       0, playlist_title);
+		       totem_pl_parser_table_signals[PLAYLIST_ENDED],
+		       0, playlist_uri);
 }
 
 static char *
@@ -687,6 +694,10 @@ totem_pl_parser_init (TotemPlParser *parser)
 				     "Abstract of the item to be added", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("moreinfo", "moreinfo",
+				     "URL to get more information for item to be added", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 	pspec = g_param_spec_string ("screensize", "screensize",
 				     "String representing the default movie size (double, full or original)", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
@@ -699,6 +710,34 @@ totem_pl_parser_init (TotemPlParser *parser)
 				     "String representing the end time of the stream", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_boolean ("is-playlist", "is-playlist",
+				      "Boolean saying whether the entry pushed is the top-level of a playlist", FALSE,
+				      G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("description", "description",
+				     "String representing the description of the stream", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("publication-date", "publication-date",
+				     "String representing the publication date of the stream", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("filesize", "filesize",
+				     "String representing the filesize of a file", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("language", "language",
+				     "String representing the language of a stream", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("contact", "contact",
+				     "String representing the contact for a playlist", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("image-url", "image-url",
+				     "String representing the location of an image for a playlist", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 }
 
 static void
@@ -729,8 +768,10 @@ totem_pl_parser_add_url_valist (TotemPlParser *parser,
 	const char *name;
 	char *title, *url;
 	GHashTable *metadata;
+	gboolean is_playlist;
 
 	title = url = NULL;
+	is_playlist = FALSE;
 
 	g_object_ref (G_OBJECT (parser));
 	metadata = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
@@ -761,8 +802,14 @@ totem_pl_parser_add_url_valist (TotemPlParser *parser,
 			break;
 		}
 
-		if (strcmp (name, "url") == 0)
+		if (strcmp (name, TOTEM_PL_PARSER_FIELD_URL) == 0)
 			url = g_value_dup_string (&value);
+		else if (strcmp (name, TOTEM_PL_PARSER_FIELD_IS_PLAYLIST) == 0) {
+			is_playlist = g_value_get_boolean (&value);
+			g_value_unset (&value);
+			name = va_arg (var_args, char*);
+			continue;
+		}
 
 		/* Ignore empty values */
 		string = g_value_get_string (&value);
@@ -793,9 +840,15 @@ totem_pl_parser_add_url_valist (TotemPlParser *parser,
 	}
 
 	if (g_hash_table_size (metadata) > 0 || url != NULL) {
-		g_signal_emit (G_OBJECT (parser),
-			       totem_pl_parser_table_signals[ENTRY_PARSED],
-			       0, url, metadata);
+		if (is_playlist == FALSE) {
+			g_signal_emit (G_OBJECT (parser),
+				       totem_pl_parser_table_signals[ENTRY_PARSED],
+				       0, url, metadata);
+		} else {
+			g_signal_emit (G_OBJECT (parser),
+				       totem_pl_parser_table_signals[PLAYLIST_STARTED],
+				       0, url, metadata);
+		}
 	}
 
 	g_hash_table_destroy (metadata);
@@ -842,7 +895,7 @@ totem_pl_parser_remove_filename (const char *url)
 }
 
 char *
-totem_pl_resolve_url (const char *base, const char *url)
+totem_pl_parser_resolve_url (const char *base, const char *url)
 {
 	GnomeVFSURI *base_uri, *new;
 	char *resolved, *base_no_frag;
@@ -909,6 +962,8 @@ static PlaylistTypes special_types[] = {
 	PLAYLIST_TYPE ("text/x-google-video-pointer", totem_pl_parser_add_gvp, NULL, FALSE),
 	PLAYLIST_TYPE ("text/google-video-pointer", totem_pl_parser_add_gvp, NULL, FALSE),
 	PLAYLIST_TYPE ("audio/x-iriver-pla", totem_pl_parser_add_pla, NULL, FALSE),
+	PLAYLIST_TYPE ("application/atom+xml", totem_pl_parser_add_atom, NULL, FALSE),
+	PLAYLIST_TYPE ("application/rss+xml", totem_pl_parser_add_rss, NULL, FALSE),
 #ifndef TOTEM_PL_PARSER_MINI
 	PLAYLIST_TYPE ("application/x-desktop", totem_pl_parser_add_desktop, NULL, TRUE),
 	PLAYLIST_TYPE ("application/x-gnome-app-info", totem_pl_parser_add_desktop, NULL, TRUE),
@@ -936,6 +991,7 @@ static PlaylistTypes dual_types[] = {
 	PLAYLIST_TYPE2 ("video/quicktime", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
 	PLAYLIST_TYPE2 ("application/x-quicktime-media-link", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
 	PLAYLIST_TYPE2 ("application/x-quicktimeplayer", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
+	PLAYLIST_TYPE2 ("application/xml", totem_pl_parser_add_xml_feed, totem_pl_parser_is_xml_feed),
 };
 
 #ifndef TOTEM_PL_PARSER_MINI
@@ -1060,6 +1116,17 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url,
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	}
 
+	/* Fix up itpc, see http://www.apple.com/itunes/store/podcaststechspecs.html */
+	if (g_str_has_prefix (url, "itpc") != FALSE) {
+		DEBUG(g_print ("URL '%s' is getting special cased for ITPC parsing\n", url));
+		return totem_pl_parser_add_itpc (parser, url, base, NULL);
+	}
+	/* Try itms Podcast references, see itunes.py in PenguinTV */
+	if (totem_pl_parser_is_itms_feed (url) != FALSE) {
+	    	DEBUG(g_print ("URL '%s' is getting special cased for ITMS parsing\n", url));
+	    	return totem_pl_parser_add_itms (parser, url, NULL, NULL);
+	}
+
 	if (!parser->priv->recurse && parser->priv->recurse_level > 0) {
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	}
@@ -1170,7 +1237,7 @@ totem_pl_parser_parse_with_base (TotemPlParser *parser, const char *url,
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
 	g_return_val_if_fail (strstr (url, "://") != NULL,
-			TOTEM_PL_PARSER_RESULT_IGNORED);
+			TOTEM_PL_PARSER_RESULT_ERROR);
 
 	parser->priv->recurse_level = 0;
 	parser->priv->fallback = fallback != FALSE;
@@ -1204,9 +1271,159 @@ totem_pl_parser_add_ignored_mimetype (TotemPlParser *parser,
 		(parser->priv->ignore_mimetypes, g_strdup (mimetype));
 }
 
-#endif /* !TOTEM_PL_PARSER_MINI */
+gint64
+totem_pl_parser_parse_duration (const char *duration, gboolean debug)
+{
+	int hours, minutes, seconds, fractions;
 
-#define D(x) if (debug) x
+	if (duration == NULL) {
+		D(g_print ("No duration passed\n"));
+		return -1;
+	}
+
+	/* Formats used by both ASX and RAM files */
+	if (sscanf (duration, "%d:%d:%d.%d", &hours, &minutes, &seconds, &fractions) == 4) {
+		gint64 ret = hours * 3600 + minutes * 60 + seconds;
+		if (ret == 0 && fractions > 0) {
+			D(g_print ("Used 00:00:00.00 format, with fractions rounding\n"));
+			ret = 1;
+		} else {
+			D(g_print ("Used 00:00:00.00 format\n"));
+		}
+		return ret;
+	}
+	if (sscanf (duration, "%d:%d:%d", &hours, &minutes, &seconds) == 3) {
+		D(g_print ("Used 00:00:00 format\n"));
+		return hours * 3600 + minutes * 60 + seconds;
+	}
+	if (sscanf (duration, "%d:%d.%d", &minutes, &seconds, &fractions) == 3) {
+		gint64 ret = minutes * 60 + seconds;
+		if (ret == 0 && fractions > 0) {
+			D(g_print ("Used 00:00.00 format, with fractions rounding\n"));
+			ret = 1;
+		} else {
+			D(g_print ("Used 00:00.00 format\n"));
+		}
+		return ret;
+	}
+	if (sscanf (duration, "%d:%d", &minutes, &seconds) == 2) {
+		D(g_print ("Used 00:00 format\n"));
+		return minutes * 60 + seconds;
+	}
+	/* PLS files format */
+	if (sscanf (duration, "%d", &seconds) == 1) {
+		D(g_print ("Used PLS format\n"));
+		return seconds;
+	}
+
+	D(g_message ("Couldn't parse duration '%s'\n", duration));
+
+	return -1;
+}
+
+guint64
+totem_pl_parser_parse_date (const char *date_str, gboolean debug)
+{
+	struct tm tm;
+	char *result;
+
+	/* RFC 2822 date format */
+	result = strptime (date_str, "%a, %d %b %Y %T", &tm);
+
+	/* same as above, but without comma */
+	if (result == NULL) {
+		memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%a %d %b %Y %T", &tm);
+	}
+
+	/* close-to-RFC 2822, but with extra 0 */
+	if (result == NULL) {
+		memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%a, %d %b %Y 0%T", &tm);
+	}
+
+	/* close-to-RFC 2822, but with no seconds */
+	if (result == NULL) {
+		memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%a, %d %b %Y %R", &tm);
+	}
+
+	/* format without weekday */
+	if (result == NULL) {
+		memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%d %b %Y %T", &tm);
+	}
+
+	/* reversed day and long month */
+	if (result == NULL) {
+		memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%a, %B %d %Y %T", &tm);
+	}
+
+	/* ISO date like */
+	if (result == NULL) {
+		memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%Y-%m-%d %T", &tm);
+	}
+
+	/* ISO date like without timezone */
+	if (result == NULL) {
+	memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%Y-%m-%d", &tm);
+	}
+
+	/* Broken weekday short names */
+	if (result == NULL) {
+		char *tmp;
+
+		/* strip off the erroneous weekday */
+		tmp = strstr (date_str, ",");
+		if (tmp != NULL) {
+			tmp++;
+			memset (&tm, 0, sizeof (struct tm));
+			result = strptime (tmp, "%d %b %Y %T", &tm);
+		}
+	}
+
+	/* format with timezone offset from GMT */
+	if (result == NULL) {
+		memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%a %b %d %T %z %Y", &tm);
+	}
+
+	/* format with timezone name */
+	if (result == NULL) {
+		char *tmp;
+
+		memset (&tm, 0, sizeof (struct tm));
+
+		/* match first part of time string */
+		result = strptime (date_str, "%a %b %d %T ", &tm);
+
+		/* look for anything with a timezone name-like format
+		   i.e. at least one all caps alphabetical character */
+		if (result != NULL) {
+			size_t n;
+
+			n = strspn(result, "ABCDEFGHIJKLMNOPQRSTUVWXYZ");
+			tmp = result+n;
+
+			/* make sure there was at least one character that matched */
+			if ((tmp != NULL) && n > 0)
+				/* remaining part must be the year */
+				result = strptime (tmp, "%Y", &tm);
+			else
+				result = NULL;
+		}
+	}
+
+	if (result == NULL)
+		D(g_message ("Couldn't parse date '%s'\n", date_str));
+
+	return (guint64) ((result == NULL) ? 0 : mktime (&tm));
+}
+
+#endif /* !TOTEM_PL_PARSER_MINI */
 
 gboolean
 totem_pl_parser_can_parse_from_data (const char *data,
