@@ -29,8 +29,8 @@
 #include "totem-private.h"
 #include "bacon-video-widget.h"
 
-#include <gst/utils/base-utils.h>
-#include <gst/utils/install-plugins.h>
+#include <gst/pbutils/pbutils.h>
+#include <gst/pbutils/install-plugins.h>
 
 #include <gst/gst.h> /* for gst_registry_update */
 
@@ -138,17 +138,28 @@ on_plugin_installation_done (GstInstallPluginsReturn res, gpointer user_data)
 				{
 					bacon_video_widget_play (ctx->totem->bvw, NULL);
 				} else {
-					/* nothing we can do, user already saw error from wizard */
+					/* wizard has not shown error, do stop/play again,
+					 * so that an error message gets shown */
 					bacon_video_widget_stop (ctx->totem->bvw);
+					bacon_video_widget_play (ctx->totem->bvw, NULL);
 				}
 			}
 			break;
 		case GST_INSTALL_PLUGINS_USER_ABORT:
 			{
-				if (ctx->playing)
+				/* blacklist on user abort, so we show an error next time (or
+				 * just play what we can) instead of calling the installer */
+				for (p = ctx->details; p != NULL && *p != NULL; ++p)
+					totem_codec_install_blacklist_plugin (*p);
+
+				if (ctx->playing) {
 					bacon_video_widget_play (ctx->totem->bvw, NULL);
-				else
+				} else {
+					/* if we couldn't play anything, do stop/play again,
+					 * so that an error message gets shown */
 					bacon_video_widget_stop (ctx->totem->bvw);
+					bacon_video_widget_play (ctx->totem->bvw, NULL);
+				}
 			}
 			break;
 		case GST_INSTALL_PLUGINS_ERROR:
@@ -267,7 +278,7 @@ totem_missing_plugins_setup (Totem *totem)
 			G_CALLBACK (totem_on_missing_plugins_event),
 			totem);
 
-	gst_base_utils_init ();
+	gst_pb_utils_init ();
 
 	GST_INFO ("Set up support for automatic missing plugin installation");
 #endif
