@@ -25,6 +25,7 @@
 #include "totem.h"
 #include "totem-private.h"
 #include "totem-session.h"
+#include "totem-uri.h"
 
 #ifndef HAVE_GTK_ONLY
 
@@ -36,12 +37,11 @@ totem_session_create_key (void)
 {
 	char *filename, *path;
 
-	filename = g_strdup_printf ("totem-%d-%d-%u.pls",
+	filename = g_strdup_printf ("playlist-%d-%d-%u.pls",
 			(int) getpid (),
 			(int) time (NULL),
 			g_random_int ());
-	path = g_build_filename (g_get_home_dir (),
-			".gnome2", filename, NULL);
+	path = g_build_filename (totem_dot_dir (), filename, NULL);
 	g_free (filename);
 
 	return path;
@@ -129,7 +129,7 @@ totem_session_restore (Totem *totem, char **filenames)
 
 	totem_signal_block_by_data (totem->playlist, totem);
 
-	if (totem_playlist_add_mrl (totem->playlist, uri, NULL) == FALSE)
+	if (totem_playlist_add_mrl_with_cursor (totem->playlist, uri, NULL) == FALSE)
 	{
 		totem_signal_unblock_by_data (totem->playlist, totem);
 		totem_action_set_mrl (totem, NULL);
