@@ -29,18 +29,6 @@
 #include "totem-private.h"
 #include "ev-sidebar.h"
 
-void
-totem_sidebar_toggle (Totem *totem)
-{
-	GtkWidget *toggle;
-	gboolean state;
-
-	toggle = glade_xml_get_widget (totem->xml, "tmw_sidebar_button");
-
-	state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggle));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), !state);
-}
-
 static void
 cb_resize (Totem * totem)
 {
@@ -53,8 +41,8 @@ cb_resize (Totem * totem)
 	h = totem->win->allocation.height;
 
 	g_value_init (&gvalue_size, G_TYPE_INT);
-	pane = glade_xml_get_widget (totem->xml, "tmw_main_pane");
-	gtk_widget_style_get_property(pane, "handle-size", &gvalue_size);
+	pane = GTK_WIDGET (gtk_builder_get_object (totem->xml, "tmw_main_pane"));
+	gtk_widget_style_get_property (pane, "handle-size", &gvalue_size);
 	handle_size = g_value_get_int (&gvalue_size);
 	
 	if (totem->sidebar_shown) {
@@ -63,25 +51,27 @@ cb_resize (Totem * totem)
 		w -= totem->sidebar->allocation.width + handle_size;
 	}
 
-	gtk_window_resize (GTK_WINDOW (totem->win), w, h);
+	if (w > 0 && h > 0)
+		gtk_window_resize (GTK_WINDOW (totem->win), w, h);
 }
 
-static void
-on_sidebar_button_toggled (GtkToggleButton *button, Totem *totem)
+void
+totem_sidebar_toggle (Totem *totem, gboolean state)
 {
-	GtkWidget *item;
-	gboolean state;
+	GtkAction *action;
+
+	if (GTK_WIDGET_VISIBLE (GTK_WIDGET (totem->sidebar)) == state)
+		return;
 
-	state = gtk_toggle_button_get_active (button);
 	if (state != FALSE)
 		gtk_widget_show (GTK_WIDGET (totem->sidebar));
 	else
 		gtk_widget_hide (GTK_WIDGET (totem->sidebar));
 
-	item = glade_xml_get_widget (totem->xml, "tmw_sidebar_menu_item");
-	totem_signal_block_by_data (G_OBJECT (item), totem);
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), state);
-	totem_signal_unblock_by_data (G_OBJECT (item), totem);
+	action = gtk_action_group_get_action (totem->main_action_group, "sidebar");
+	totem_signal_block_by_data (G_OBJECT (action), totem);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), state);
+	totem_signal_unblock_by_data (G_OBJECT (action), totem);
 
 	gconf_client_set_bool (totem->gc,
 				GCONF_PREFIX"/sidebar_shown",
@@ -94,10 +84,7 @@ on_sidebar_button_toggled (GtkToggleButton *button, Totem *totem)
 static void
 toggle_sidebar_from_sidebar (GtkWidget *playlist, Totem *totem)
 {
-	GtkWidget *button;
-
-	button = glade_xml_get_widget (totem->xml, "tmw_sidebar_button");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
+	totem_sidebar_toggle (totem, FALSE);
 }
 
 gboolean
@@ -109,16 +96,14 @@ totem_sidebar_is_visible (Totem *totem)
 void
 totem_sidebar_setup (Totem *totem, gboolean visible, const char *page_id)
 {
-	GtkWidget *item, *item2;
+	GtkPaned *item;
+	GtkAction *action;
 
-	item = glade_xml_get_widget (totem->xml, "tmw_main_pane");
+	item = GTK_PANED (gtk_builder_get_object (totem->xml, "tmw_main_pane"));
 	totem->sidebar = ev_sidebar_new ();
 	ev_sidebar_add_page (EV_SIDEBAR (totem->sidebar),
 			"playlist", _("Playlist"),
 			GTK_WIDGET (totem->playlist));
-	ev_sidebar_add_page (EV_SIDEBAR (totem->sidebar),
-			"properties", _("Properties"),
-			GTK_WIDGET (totem->properties));
 	if (page_id != NULL) {
 		ev_sidebar_set_current_page (EV_SIDEBAR (totem->sidebar),
 				page_id);
@@ -126,26 +111,21 @@ totem_sidebar_setup (Totem *totem, gboolean visible, const char *page_id)
 		ev_sidebar_set_current_page (EV_SIDEBAR (totem->sidebar),
 				"playlist");
 	}
-	gtk_widget_set_sensitive (GTK_WIDGET (totem->properties), FALSE);
-	gtk_paned_pack2 (GTK_PANED (item), totem->sidebar, FALSE, FALSE);
+	gtk_paned_pack2 (item, totem->sidebar, FALSE, FALSE);
 
 	totem->sidebar_shown = visible;
 
-	item = glade_xml_get_widget (totem->xml, "tmw_sidebar_button");
-	item2 = glade_xml_get_widget (totem->xml, "tmw_sidebar_menu_item");
+	action = gtk_action_group_get_action (totem->main_action_group,
+			"sidebar");
 
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (item), visible);
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item2), visible);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
 
 	/* Signals */
-	g_signal_connect (G_OBJECT (item), "clicked",
-			G_CALLBACK (on_sidebar_button_toggled), totem);
 	g_signal_connect (G_OBJECT (totem->sidebar), "closed",
 			G_CALLBACK (toggle_sidebar_from_sidebar), totem);
-	g_signal_connect_swapped (G_OBJECT (item2), "activate",
-			G_CALLBACK (totem_sidebar_toggle), totem);
 
 	gtk_widget_show_all (totem->sidebar);
+	gtk_widget_realize (totem->sidebar);
 
 	if (!visible)
 		gtk_widget_hide (totem->sidebar);
@@ -156,3 +136,11 @@ totem_sidebar_get_current_page (Totem *totem)
 {
 	return ev_sidebar_get_current_page (EV_SIDEBAR (totem->sidebar));
 }
+
+void
+totem_sidebar_set_current_page (Totem *totem, const char *name)
+{
+	ev_sidebar_set_current_page (EV_SIDEBAR (totem->sidebar), name);
+	totem_sidebar_toggle (totem, TRUE);
+}
+
