@@ -83,6 +83,9 @@
 #define ZOOM_DISABLE (ZOOM_LOWER - 1)
 #define ZOOM_ENABLE (ZOOM_UPPER + 1)
 
+#define DEFAULT_WINDOW_W 650
+#define DEFAULT_WINDOW_H 500
+
 #define VOLUME_EPSILON (1e-10)
 
 #define BVW_VBOX_BORDER_WIDTH 1
@@ -328,29 +331,20 @@ play_pause_set_label (Totem *totem, TotemStates state)
 void
 totem_action_eject (Totem *totem)
 {
-	GError *err = NULL;
-	char *cmd, *prefix;
-	const char *needle;
-	char *device;
+	GnomeVFSVolume *volume;
 
-	needle = strchr (totem->mrl, ':');
-	g_assert (needle != NULL);
-	/* we want the ':' as well */
-	prefix = g_strndup (totem->mrl, needle - totem->mrl + 1);
-	totem_playlist_clear_with_prefix (totem->playlist, prefix);
-	g_free (prefix);
+	volume = totem_get_volume_for_media (totem->mrl);
+	if (volume == NULL)
+		return;
 
-	g_object_get (G_OBJECT (totem->bvw),
-			"mediadev", &device, NULL);
-	cmd = g_strdup_printf ("eject %s", device);
-	g_free (device);
+	g_free (totem->mrl);
+	totem->mrl = NULL;
+	bacon_video_widget_close (totem->bvw);
+	totem_file_closed (totem);
 
-	if (g_spawn_command_line_sync (cmd, NULL, NULL, NULL, &err) == FALSE)
-	{
-		totem_action_error (_("Totem could not eject the optical media."), err->message, totem);
-		g_error_free (err);
-	}
-	g_free (cmd);
+	/* the volume monitoring will take care of removing the items */
+	gnome_vfs_volume_eject (volume, NULL, NULL);
+	gnome_vfs_volume_unref (volume);
 }
 
 void
@@ -410,8 +404,15 @@ totem_action_seek (Totem *totem, double pos)
 		msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
 		g_free (disp);
 
-		totem_action_error (msg, err->message, totem);
+		/* Release the lock and reset everything so that we
+		 * avoid being "stuck" seeking */
+		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), FALSE);
+		totem_time_label_set_seeking (TOTEM_TIME_LABEL (totem->fs->time_label), FALSE);
+		totem->seek_lock = FALSE;
+		bacon_video_widget_seek (totem->bvw, 0, NULL);
 		totem_action_stop (totem);
+
+		totem_action_error (msg, err->message, totem);
 		g_free (msg);
 		g_error_free (err);
 	}
@@ -466,8 +467,13 @@ totem_action_load_media (Totem *totem, TotemDiscMediaType type, const char *devi
 	gboolean retval;
 
 	if (bacon_video_widget_can_play (totem->bvw, type) == FALSE) {
-		msg = g_strdup_printf (_("Totem cannot play this type of media (%s) because you do not have the appropriate plugins to handle it."), _(totem_cd_get_human_readable_name (type)));
-		totem_action_error (msg, _("Please install the necessary plugins and restart Totem to be able to play this media."), totem);
+		if (type == MEDIA_TYPE_DVD || type == MEDIA_TYPE_VCD)
+			msg = g_strdup_printf(_("Totem cannot play this type of media (%s) because it does not have the appropriate plugins to be able to read from the disc."), _(totem_cd_get_human_readable_name (type)));
+		else
+			msg = g_strdup_printf (_("Totem cannot play this type of media (%s) because you do not have the appropriate plugins to handle it."), _(totem_cd_get_human_readable_name (type)));
+		totem_interface_error_with_link (msg, _("Please install the necessary plugins and restart Totem to be able to play this media."),
+				"http://www.gnome.org/projects/totem/#codecs", _("More information about media plugins"),
+				GTK_WINDOW (totem->win), totem);
 		g_free (msg);
 		return FALSE;
 	}
@@ -515,12 +521,20 @@ totem_action_load_media_device (Totem *totem, const char *device)
 			break;
 		case MEDIA_TYPE_DVD:
 		case MEDIA_TYPE_VCD:
-		case MEDIA_TYPE_CDDA:
-			retval = totem_action_load_media (totem, type, device_path);
-			if (retval == FALSE) {
-				totem_action_set_mrl_and_play (totem, NULL);
+			{
+				const char *filenames[2];
+
+				filenames[0] = url;
+				filenames[1] = NULL;
+
+				retval = totem_action_open_files (totem, (char **) filenames);
 			}
 			break;
+		case MEDIA_TYPE_CDDA:
+			totem_action_error (_("Totem does not support playback of Audio CDs"),
+					    _("Please consider using a music player or a CD extractor to play this CD"),
+					    totem);
+			retval = FALSE;
 		default:
 			g_assert_not_reached ();
 	}
@@ -540,7 +554,7 @@ totem_action_play_media_device (Totem *totem, const char *device)
 		mrl = totem_playlist_get_current_mrl (totem->playlist);
 		totem_action_set_mrl_and_play (totem, mrl);
 		g_free (mrl);
-	} 
+	}
 }
 
 void
@@ -827,7 +841,7 @@ update_mrl_label (Totem *totem, const char *name)
 		/* Get the length of the stream */
 		time = bacon_video_widget_get_stream_length (totem->bvw);
 		totem_statusbar_set_time_and_length (TOTEM_STATUSBAR
-				(totem->statusbar), 0, time / 1000);
+						     (totem->statusbar), 0, time / 1000);
 
 		g_object_notify (G_OBJECT (totem), "stream-length");
 
@@ -900,12 +914,13 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 	} else {
 		gboolean caps;
 		gdouble volume;
-		char *subtitle_uri;
+		char *subtitle_uri = NULL;
 		GError *err = NULL;
 
 		bacon_video_widget_set_logo_mode (totem->bvw, FALSE);
 
-		subtitle_uri = totem_uri_get_subtitle_uri (mrl);
+		if (totem->autoload_subs != FALSE)
+			subtitle_uri = totem_uri_get_subtitle_uri (mrl);
 		totem_gdk_window_set_waiting_cursor (totem->win->window);
 		retval = bacon_video_widget_open_with_subtitle (totem->bvw,
 				mrl, subtitle_uri, &err);
@@ -1026,23 +1041,33 @@ totem_action_next (Totem *totem)
 	totem_action_direction (totem, TOTEM_PLAYLIST_DIRECTION_NEXT);
 }
 
-void
-totem_action_seek_relative (Totem *totem, int off_sec)
+static void
+totem_seek_time_rel (Totem *totem, gint64 time, gboolean relative)
 {
 	GError *err = NULL;
-	gint64 off_msec, oldsec, sec;
+	gint64 sec;
 
 	if (totem->mrl == NULL)
 		return;
 	if (bacon_video_widget_is_seekable (totem->bvw) == FALSE)
 		return;
 
-	off_msec = off_sec * 1000;
-	oldsec = bacon_video_widget_get_current_time (totem->bvw);
-	sec = MAX (0, oldsec + off_msec);
+	totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), TRUE);
+	totem_time_label_set_seeking (TOTEM_TIME_LABEL (totem->fs->time_label), TRUE);
+
+	if (relative != FALSE) {
+		gint64 oldmsec;
+		oldmsec = bacon_video_widget_get_current_time (totem->bvw);
+		sec = MAX (0, oldmsec + time);
+	} else {
+		sec = time;
+	}
 
 	bacon_video_widget_seek_time (totem->bvw, sec, &err);
 
+	totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), FALSE);
+	totem_time_label_set_seeking (TOTEM_TIME_LABEL (totem->fs->time_label), FALSE);
+
 	if (err != NULL)
 	{
 		char *msg, *disp;
@@ -1058,31 +1083,16 @@ totem_action_seek_relative (Totem *totem, int off_sec)
 	}
 }
 
+void
+totem_action_seek_relative (Totem *totem, gint64 offset)
+{
+	totem_seek_time_rel (totem, offset, TRUE);
+}
+
 void
 totem_action_seek_time (Totem *totem, gint64 sec)
 {
-	GError *err = NULL;
-
-	if (totem->mrl == NULL)
-		return;
-	if (bacon_video_widget_is_seekable (totem->bvw) == FALSE)
-		return;
-
-	bacon_video_widget_seek_time (totem->bvw, sec, &err);
-
-	if (err != NULL)
-	{
-		char *msg, *disp;
-
-		disp = totem_uri_escape_for_display (totem->mrl);
-		msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
-		g_free (disp);
-
-		totem_action_stop (totem);
-		totem_action_error (msg, err->message, totem);
-		g_free (msg);
-		g_error_free (err);
-	}
+	totem_seek_time_rel (totem, sec, FALSE);
 }
 
 static void
@@ -1219,6 +1229,8 @@ totem_action_drop_files (Totem *totem, GtkSelectionData *data,
 	if (file_list == NULL)
 		return FALSE;
 
+	totem_gdk_window_set_waiting_cursor (totem->win->window);
+
 	if (drop_type != 1)
 		file_list = g_list_sort (file_list, (GCompareFunc) strcmp);
 	else
@@ -1232,6 +1244,8 @@ totem_action_drop_files (Totem *totem, GtkSelectionData *data,
 			continue;
 
 		filename = totem_create_full_path (p->data);
+		if (filename == NULL)
+			filename = g_strdup (p->data);
 		title = NULL;
 
 		if (empty_pl != FALSE && cleared == FALSE)
@@ -1259,13 +1273,14 @@ totem_action_drop_files (Totem *totem, GtkSelectionData *data,
 			}
 		}
 
-		totem_playlist_add_mrl (totem->playlist, filename ? filename : p->data, title);
+		totem_playlist_add_mrl (totem->playlist, filename, title);
 
 		g_free (filename);
 		g_free (p->data);
 	}
 
 	g_list_free (file_list);
+	gdk_window_set_cursor (totem->win->window, NULL);
 
 	/* ... and reconnect because we're nice people */
 	if (cleared != FALSE)
@@ -1354,7 +1369,7 @@ on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
 	gchar *old_mrl, *new_mrl;
 
 	old_mrl = totem_playlist_get_current_mrl (TOTEM_PLAYLIST (totem->playlist));
-	new_mrl = totem_resolve_relative_link (old_mrl, mrl);
+	new_mrl = totem_pl_parser_resolve_url (old_mrl, mrl);
 	g_free (old_mrl);
 
 	bacon_video_widget_close (totem->bvw);
@@ -1367,14 +1382,14 @@ on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
 	g_free (new_mrl);
 }
 
-/* This is only called when we are playing a DVD */
+/* This is only called when we are playing a DVD or a radio */
 static void
 on_title_change_event (BaconVideoWidget *bvw, const char *string, Totem *totem)
 {
 	update_mrl_label (totem, string);
 	update_buttons (totem);
 	totem_playlist_set_title (TOTEM_PLAYLIST (totem->playlist),
-			string, TRUE);
+				  string, TRUE);
 }
 
 static void
@@ -1423,7 +1438,7 @@ static void
 on_got_metadata_event (BaconVideoWidget *bvw, Totem *totem)
 {
         char *name = NULL;
-
+	
 	totem_metadata_updated (totem, NULL, NULL, NULL);
 
 	name = totem_get_nice_name_for_stream (totem);
@@ -1433,8 +1448,11 @@ on_got_metadata_event (BaconVideoWidget *bvw, Totem *totem)
 			(TOTEM_PLAYLIST (totem->playlist), name, FALSE);
 		g_free (name);
 	}
-	on_playlist_change_name 	 
-		(TOTEM_PLAYLIST (totem->playlist), totem);
+	
+	totem_action_set_sensitivity ("take-screenshot",
+				      bacon_video_widget_can_get_frames (bvw, NULL));
+	
+	on_playlist_change_name (TOTEM_PLAYLIST (totem->playlist), totem);
 }
 
 static void
@@ -1461,6 +1479,8 @@ on_buffering_event (BaconVideoWidget *bvw, int percentage, Totem *totem)
 static void
 update_seekable (Totem *totem)
 {
+	GtkAction *action;
+	GtkActionGroup *action_group;
 	gboolean seekable;
 
 	seekable = bacon_video_widget_is_seekable (totem->bvw);
@@ -1475,8 +1495,17 @@ update_seekable (Totem *totem)
 
 	totem_fullscreen_set_seekable (totem->fs, seekable);
 
-	totem_action_set_sensitivity ("skip-forward", seekable);
-	totem_action_set_sensitivity ("skip-backwards", seekable);
+	/* FIXME: We can use this code again once bug #457631 is fixed and
+	 * skip-* are back in the main action group. */
+	/*totem_action_set_sensitivity ("skip-forward", seekable);
+	totem_action_set_sensitivity ("skip-backwards", seekable);*/
+	action_group = GTK_ACTION_GROUP (gtk_builder_get_object (totem->xml, "skip-action-group"));
+
+	action = gtk_action_group_get_action (action_group, "skip-forward");
+	gtk_action_set_sensitive (action, seekable);
+
+	action = gtk_action_group_get_action (action_group, "skip-backwards");
+	gtk_action_set_sensitive (action, seekable);
 
 	g_object_notify (G_OBJECT (totem), "seekable");
 }
@@ -1485,7 +1514,7 @@ static void
 update_current_time (BaconVideoWidget *bvw,
 		gint64 current_time,
 		gint64 stream_length,
-		float current_position,
+		double current_position,
 		gboolean seekable, Totem *totem)
 {
 	if (totem->seek_lock == FALSE)
@@ -1607,9 +1636,7 @@ seek_slider_released_cb (GtkWidget *widget, GdkEventButton *event, Totem *totem)
 	val = gtk_adjustment_get_value (adj);
 
 	if (bacon_video_widget_can_direct_seek (totem->bvw) == FALSE)
-	{
 		totem_action_seek (totem, val / 65535.0);
-	}
 
 	totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), FALSE);
 	totem_time_label_set_seeking (TOTEM_TIME_LABEL (totem->fs->time_label),
@@ -1646,6 +1673,8 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 	if (list == NULL)
 		return changed;
 
+	totem_gdk_window_set_waiting_cursor (totem->win->window);
+
 	for (l = list ; l != NULL; l = l->next)
 	{
 		char *filename;
@@ -1668,8 +1697,6 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 				|| strstr (filename, "://") != NULL
 				|| g_str_has_prefix (filename, "dvd:") != FALSE
 				|| g_str_has_prefix (filename, "vcd:") != FALSE
-				|| g_str_has_prefix (filename, "cdda:") != FALSE
-				|| g_str_has_prefix (filename, "cd:") != FALSE
 				|| g_str_has_prefix (filename, "dvb:") != FALSE)
 		{
 			if (cleared == FALSE)
@@ -1689,16 +1716,12 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 			if (totem_is_block_device (filename) != FALSE) {
 				totem_action_load_media_device (totem, data);
 				changed = TRUE;
-			} else if (g_str_has_prefix (filename, "cdda:/") != FALSE) {
-				totem_playlist_add_mrl (totem->playlist, data, NULL);
-				changed = TRUE;
 			} else if (g_str_has_prefix (filename, "dvb:/") != FALSE) {
 				totem_playlist_add_mrl (totem->playlist, data, NULL);
 				changed = TRUE;
 			} else if (g_str_equal (filename, "dvb:") != FALSE) {
-				totem_action_load_media (totem, MEDIA_TYPE_DVB, NULL);
-			} else if (totem_playlist_add_mrl (totem->playlist,
-						filename, NULL) != FALSE) {
+				changed = totem_action_load_media (totem, MEDIA_TYPE_DVB, NULL);
+			} else if (totem_playlist_add_mrl (totem->playlist, filename, NULL) != FALSE) {
 				totem_action_add_recent (totem, filename);
 				changed = TRUE;
 			}
@@ -1707,6 +1730,8 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 		g_free (filename);
 	}
 
+	gdk_window_set_cursor (totem->win->window, NULL);
+
 	/* ... and reconnect because we're nice people */
 	if (cleared != FALSE)
 	{
@@ -1855,11 +1880,10 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		totem_action_pause (totem);
 		break;
 	case TOTEM_REMOTE_COMMAND_SEEK_FORWARD:
-		totem_action_seek_relative (totem, SEEK_FORWARD_OFFSET);
+		totem_action_seek_relative (totem, SEEK_FORWARD_OFFSET * 1000);
 		break;
 	case TOTEM_REMOTE_COMMAND_SEEK_BACKWARD:
-		totem_action_seek_relative (totem,
-				SEEK_BACKWARD_OFFSET);
+		totem_action_seek_relative (totem, SEEK_BACKWARD_OFFSET * 1000);
 		break;
 	case TOTEM_REMOTE_COMMAND_VOLUME_UP:
 		totem_action_volume_relative (totem, VOLUME_UP_OFFSET);
@@ -1881,7 +1905,7 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		break;
 	case TOTEM_REMOTE_COMMAND_ENQUEUE:
 		g_assert (url != NULL);
-		if (totem_playlist_add_mrl (totem->playlist, url, NULL) != FALSE) {
+		if (totem_playlist_add_mrl_with_cursor (totem->playlist, url, NULL) != FALSE) {
 			totem_action_add_recent (totem, url);
 		}
 		break;
@@ -1890,7 +1914,7 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		if (url == NULL) {
 			bacon_video_widget_close (totem->bvw);
 			totem_file_closed (totem);
-			totem_action_set_mrl_and_play (totem, NULL);
+			totem_action_set_mrl (totem, NULL);
 			break;
 		}
 		if (strcmp (url, "dvd:") == 0) {
@@ -1899,15 +1923,9 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		} else if (strcmp (url, "vcd:") == 0) {
 			//FIXME b0rked
 			totem_action_play_media (totem, MEDIA_TYPE_VCD, NULL);
-		} else if (g_str_has_prefix (url, "cd:") != FALSE) {
-			//FIXME b0rked
-			totem_action_play_media (totem, MEDIA_TYPE_CDDA, NULL);
 		} else if (g_str_has_prefix (url, "dvb:") != FALSE) {
 			totem_action_play_media (totem, MEDIA_TYPE_DVB, NULL);
-		} else if (g_str_has_prefix (url, "cdda:/") != FALSE) {
-			totem_playlist_add_mrl (totem->playlist, url, NULL);
-		} else if (totem_playlist_add_mrl (totem->playlist,
-					url, NULL) != FALSE) {
+		} else if (totem_playlist_add_mrl_with_cursor (totem->playlist, url, NULL) != FALSE) {
 			totem_action_add_recent (totem, url);
 		}
 		break;
@@ -2057,18 +2075,8 @@ playlist_changed_cb (GtkWidget *playlist, Totem *totem)
 	if (mrl == NULL)
 		return;
 
-	//FIXME we shouldn't compare the 2 URLs
-	// http://bugzilla.gnome.org/show_bug.cgi?id=364311
-	if (totem->mrl == NULL
-			|| (totem->mrl != NULL && mrl != NULL
-			&& strcmp (totem->mrl, mrl) != 0))
-	{
+	if (totem_playlist_get_playing (totem->playlist) == TOTEM_PLAYLIST_STATUS_NONE)
 		totem_action_set_mrl_and_play (totem, mrl);
-	} else if (totem->mrl != NULL) {
-		//FIXME that's busted
-		totem_playlist_set_playing (totem->playlist, TOTEM_PLAYLIST_STATUS_PLAYING);
-	}
-
 	g_free (mrl);
 }
 
@@ -2228,8 +2236,8 @@ totem_action_handle_key_release (Totem *totem, GdkEventKey *event)
 	switch (event->keyval) {
 	case GDK_Left:
 	case GDK_Right:
-		totem_statusbar_set_seeking
-			(TOTEM_STATUSBAR (totem->statusbar), FALSE);
+		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), FALSE);
+		totem_time_label_set_seeking (TOTEM_TIME_LABEL (totem->fs->time_label), FALSE);
 		break;
 	}
 
@@ -2240,41 +2248,26 @@ static void
 totem_action_handle_seek (Totem *totem, GdkEventKey *event, gboolean is_forward)
 {
 	if (is_forward != FALSE) {
-		if (event->state & GDK_SHIFT_MASK) {
-			totem_action_seek_relative (totem,
-					SEEK_FORWARD_SHORT_OFFSET);
-		} else if (event->state & GDK_CONTROL_MASK) {
-			totem_action_seek_relative (totem,
-					SEEK_FORWARD_LONG_OFFSET);
-		} else {
-			totem_action_seek_relative (totem,
-					SEEK_FORWARD_OFFSET);
-		}
+		if (event->state & GDK_SHIFT_MASK)
+			totem_action_seek_relative (totem, SEEK_FORWARD_SHORT_OFFSET * 1000);
+		else if (event->state & GDK_CONTROL_MASK)
+			totem_action_seek_relative (totem, SEEK_FORWARD_LONG_OFFSET * 1000);
+		else
+			totem_action_seek_relative (totem, SEEK_FORWARD_OFFSET * 1000);
 	} else {
-		if (event->state & GDK_SHIFT_MASK) {
-			totem_action_seek_relative (totem,
-					SEEK_BACKWARD_SHORT_OFFSET);
-		} else if (event->state & GDK_CONTROL_MASK) {
-			totem_action_seek_relative (totem,
-					SEEK_BACKWARD_LONG_OFFSET);
-		} else {
-			totem_action_seek_relative (totem,
-					SEEK_BACKWARD_OFFSET);
-		}
+		if (event->state & GDK_SHIFT_MASK)
+			totem_action_seek_relative (totem, SEEK_BACKWARD_SHORT_OFFSET * 1000);
+		else if (event->state & GDK_CONTROL_MASK)
+			totem_action_seek_relative (totem, SEEK_BACKWARD_LONG_OFFSET * 1000);
+		else
+			totem_action_seek_relative (totem, SEEK_BACKWARD_OFFSET * 1000);
 	}
 }
 
 static gboolean
 totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 {
-	gboolean retval = TRUE, playlist_focused = FALSE;
-	GtkWidget *focused;
-
-	focused = gtk_window_get_focus (GTK_WINDOW (totem->win));
-	if (focused != NULL && gtk_widget_is_ancestor
-			(focused, GTK_WIDGET (totem->playlist)) != FALSE) {
-		playlist_focused = TRUE;
-	}
+	gboolean retval = TRUE;
 
 	switch (event->keyval) {
 	case GDK_A:
@@ -2405,22 +2398,12 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 			totem_action_fullscreen (totem, FALSE);
 		break;
 	case GDK_Left:
-		if (playlist_focused != FALSE)
-			return FALSE;
-
-		totem_statusbar_set_seeking
-			(TOTEM_STATUSBAR (totem->statusbar), TRUE);
 		if (gtk_widget_get_direction (totem->win) == GTK_TEXT_DIR_RTL)
 			totem_action_handle_seek (totem, event, TRUE);
 		else
 			totem_action_handle_seek (totem, event, FALSE);
 		break;
 	case GDK_Right:
-		if (playlist_focused != FALSE)
-			return FALSE;
-
-		totem_statusbar_set_seeking
-			(TOTEM_STATUSBAR (totem->statusbar), TRUE);
 		if (gtk_widget_get_direction (totem->win) == GTK_TEXT_DIR_RTL)
 			totem_action_handle_seek (totem, event, FALSE);
 		else
@@ -2433,13 +2416,9 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 			retval = FALSE;
 		break;
 	case GDK_Up:
-		if (playlist_focused != FALSE)
-			return FALSE;
 		totem_action_volume_relative (totem, VOLUME_UP_OFFSET);
 		break;
 	case GDK_Down:
-		if (playlist_focused != FALSE)
-			return FALSE;
 		totem_action_volume_relative (totem, VOLUME_DOWN_OFFSET);
 		break;
 	case GDK_0:
@@ -2458,13 +2437,9 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 		totem_action_set_scale_ratio (totem, 2);
 		break;
 	case GDK_Menu:
-		if (playlist_focused != FALSE)
-			return FALSE;
 		totem_action_menu_popup (totem, 0);
 		break;
 	case GDK_F10:
-		if (playlist_focused != FALSE)
-			return FALSE;
 		if (!(event->state & GDK_SHIFT_MASK))
 			return FALSE;
 
@@ -2487,23 +2462,27 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 	case GDK_KP_Up:
 	case GDK_KP_8:
 		bacon_video_widget_dvd_event (totem->bvw, 
-				BVW_DVD_ROOT_MENU_UP);
+					      BVW_DVD_ROOT_MENU_UP);
 		break;
 	case GDK_KP_Down:
 	case GDK_KP_2:
 		bacon_video_widget_dvd_event (totem->bvw, 
-				BVW_DVD_ROOT_MENU_DOWN);
+					      BVW_DVD_ROOT_MENU_DOWN);
 		break;
 	case GDK_KP_Right:
 	case GDK_KP_6:
 		bacon_video_widget_dvd_event (totem->bvw, 
-				BVW_DVD_ROOT_MENU_RIGHT);
+					      BVW_DVD_ROOT_MENU_RIGHT);
 		break;
 	case GDK_KP_Left:
 	case GDK_KP_4:
 		bacon_video_widget_dvd_event (totem->bvw, 
-				BVW_DVD_ROOT_MENU_LEFT);
+					      BVW_DVD_ROOT_MENU_LEFT);
 		break;
+	case GDK_KP_Begin:
+	case GDK_KP_5:
+		bacon_video_widget_dvd_event (totem->bvw,
+					      BVW_DVD_ROOT_MENU_SELECT);
 	default:
 		retval = FALSE;
 	}
@@ -2520,12 +2499,10 @@ totem_action_handle_scroll (Totem *totem, GdkScrollDirection direction)
 
 	switch (direction) {
 	case GDK_SCROLL_UP:
-		totem_action_seek_relative
-			(totem, SEEK_FORWARD_SHORT_OFFSET);
+		totem_action_seek_relative (totem, SEEK_FORWARD_SHORT_OFFSET * 1000);
 		break;
 	case GDK_SCROLL_DOWN:
-		totem_action_seek_relative
-			(totem, SEEK_BACKWARD_SHORT_OFFSET);
+		totem_action_seek_relative (totem, SEEK_BACKWARD_SHORT_OFFSET * 1000);
 		break;
 	default:
 		retval = FALSE;
@@ -2537,6 +2514,14 @@ totem_action_handle_scroll (Totem *totem, GdkScrollDirection direction)
 int
 window_key_press_event_cb (GtkWidget *win, GdkEventKey *event, Totem *totem)
 {
+	GtkWidget *focused;
+
+	focused = gtk_window_get_focus (GTK_WINDOW (totem->win));
+	if (focused != NULL && gtk_widget_is_ancestor
+			(focused, GTK_WIDGET (totem->sidebar)) != FALSE) {
+		return FALSE;
+	}
+
 	/* Special case Eject, Open, Open URI and
 	 * seeking keyboard shortcuts */
 	if (event->state != 0
@@ -2606,7 +2591,8 @@ window_scroll_event_cb (GtkWidget *win, GdkEventScroll *event, Totem *totem)
 static void
 update_media_menu_items (Totem *totem)
 {
-        gboolean playing;
+	GnomeVFSVolume *volume;
+	gboolean playing;
 
 	playing = totem_playing_dvd (totem->mrl);
 
@@ -2618,8 +2604,10 @@ update_media_menu_items (Totem *totem)
 	/* FIXME we should only show that if we have multiple angles */
 	totem_action_set_sensitivity ("next-angle", playing);
 
-	playing = totem_is_media (totem->mrl);
-	totem_action_set_sensitivity ("eject", playing);
+	volume = totem_get_volume_for_media (totem->mrl);
+	totem_action_set_sensitivity ("eject", volume != NULL);
+	if (volume != NULL)
+		gnome_vfs_volume_unref (volume);
 }
 
 static void
@@ -2675,7 +2663,9 @@ totem_setup_window (Totem *totem)
 	keyfile = g_key_file_new ();
 	if (g_key_file_load_from_file (keyfile, filename,
 			G_KEY_FILE_NONE, NULL) == FALSE) {
-		totem->sidebar_w = w = h = 0;
+		totem->sidebar_w = 0;
+		w = DEFAULT_WINDOW_W;
+		h = DEFAULT_WINDOW_H;
 		show_sidebar = TRUE;
 		page_id = NULL;
 		g_free (filename);
@@ -2752,6 +2742,7 @@ totem_callback_connect (Totem *totem)
 {
 	GtkWidget *item, *arrow;
 	GtkAction *action;
+	GtkActionGroup *action_group;
 	GtkBox *box;
 
 	/* Menu items */
@@ -2768,14 +2759,6 @@ totem_callback_connect (Totem *totem)
 	/* Controls */
 	box = GTK_BOX (gtk_builder_get_object (totem->xml, "tmw_buttons_hbox"));
 
-	action = gtk_action_group_get_action (totem->main_action_group, "play");
-	item = gtk_action_create_tool_item (action);
-	atk_object_set_name (gtk_widget_get_accessible (item),
-			_("Play / Pause"));
-	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (item),
- 					_("Play / Pause"));
-	gtk_box_pack_start (box, item, FALSE, FALSE, 0);
-
 	action = gtk_action_group_get_action (totem->main_action_group,
 			"previous-chapter");
 	item = gtk_action_create_tool_item (action);
@@ -2785,6 +2768,14 @@ totem_callback_connect (Totem *totem)
 			_("Previous Chapter/Movie"));
 	gtk_box_pack_start (box, item, FALSE, FALSE, 0);
 
+	action = gtk_action_group_get_action (totem->main_action_group, "play");
+	item = gtk_action_create_tool_item (action);
+	atk_object_set_name (gtk_widget_get_accessible (item),
+			_("Play / Pause"));
+	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (item),
+ 					_("Play / Pause"));
+	gtk_box_pack_start (box, item, FALSE, FALSE, 0);
+
 	action = gtk_action_group_get_action (totem->main_action_group,
 			"next-chapter");
 	item = gtk_action_create_tool_item (action);
@@ -2839,12 +2830,28 @@ totem_callback_connect (Totem *totem)
 	gtk_widget_add_events (totem->seek, GDK_SCROLL_MASK);
 	gtk_widget_add_events (totem->fs->seek, GDK_SCROLL_MASK);
 
+	/* FIXME Hack to fix bug #462286 */
+	g_signal_connect (G_OBJECT (totem->fs->seek), "button-press-event",
+			G_CALLBACK (seek_slider_pressed_cb), totem);
+	g_signal_connect (G_OBJECT (totem->fs->seek), "button-release-event",
+			G_CALLBACK (seek_slider_released_cb), totem);
+
 	/* Set sensitivity of the toolbar buttons */
 	totem_action_set_sensitivity ("play", FALSE);
 	totem_action_set_sensitivity ("next-chapter", FALSE);
 	totem_action_set_sensitivity ("previous-chapter", FALSE);
-	totem_action_set_sensitivity ("skip-forward", FALSE);
-	totem_action_set_sensitivity ("skip-backwards", FALSE);
+	/* FIXME: We can use this code again once bug #457631 is fixed
+	 * and skip-* are back in the main action group. */
+	/*totem_action_set_sensitivity ("skip-forward", FALSE);
+	totem_action_set_sensitivity ("skip-backwards", FALSE);*/
+
+	action_group = GTK_ACTION_GROUP (gtk_builder_get_object (totem->xml, "skip-action-group"));
+
+	action = gtk_action_group_get_action (action_group, "skip-forward");
+	gtk_action_set_sensitive (action, FALSE);
+
+	action = gtk_action_group_get_action (action_group, "skip-backwards");
+	gtk_action_set_sensitive (action, FALSE);
 }
 
 static void
@@ -3006,18 +3013,6 @@ totem_message_connection_receive_cb (const char *msg, Totem *totem)
 	g_free (url);
 }
 
-GtkWidget *
-totem_volume_create (void)
-{
-	GtkWidget *widget;
-
-	widget = gtk_volume_button_new ();
-	gtk_widget_set_sensitive (widget, FALSE);
-	gtk_widget_show (widget);
-
-	return widget;
-}
-
 int
 main (int argc, char **argv)
 {
@@ -3045,6 +3040,7 @@ main (int argc, char **argv)
 #endif
 
 	g_thread_init (NULL);
+	g_type_init ();
 
 	/* Handle command line arguments */
 	context = g_option_context_new (N_("- Play movies and songs"));
